--+-------------------------------------------------------------------------------------------------+
--|																									|
--|  File:			pcidec.vhd                                                          			|
--|																									|
--|  Project:		pci32tlite_oc																	|
--|																									|
--|  Description: 	PCI decoder and PCI signals loader.												|
--|					* LoaD signals: "ad" -> adr, cbe -> cmd.                                 		|
--|					* Decode memory and configuration space. 										|
--|	 																								|
--+-------------------------------------------------------------------------------------------------+
--|																									|
--|  Revision history :																				|
--|  Date 		  Version	Author	Description														|
--|  2005-05-13   R00A00	PAU		First alfa revision (eng)										|
--|																									|
--|  To do:		 																					|
--|																									|
--+-------------------------------------------------------------------------------------------------+
--+-----------------------------------------------------------------+
--| 																|
--|  Copyright (C) 2005 Peio Azkarate, peio@opencores.org   		| 
--| 																|
--|  This source file may be used and distributed without     		|
--|  restriction provided that this copyright statement is not		|
--|  removed from the file and that any derivative work contains	|
--|  the original copyright notice and the associated disclaimer.	|
--|                                                              	|
--|  This source file is free software; you can redistribute it     |
--|  and/or modify it under the terms of the GNU Lesser General     |
--|  Public License as published by the Free Software Foundation;   |
--|  either version 2.1 of the License, or (at your option) any     |
--|  later version.                                                 |
--| 																|
--|  This source is distributed in the hope that it will be         |
--|  useful, but WITHOUT ANY WARRANTY; without even the implied     |
--|  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR        |
--|  PURPOSE.  See the GNU Lesser General Public License for more   |
--|  details.                                                       |
--| 																|
--|  You should have received a copy of the GNU Lesser General      |
--|  Public License along with this source; if not, download it     |
--|  from http://www.opencores.org/lgpl.shtml                       |
--| 																|
--+-----------------------------------------------------------------+ 


--+-----------------------------------------------------------------------------+
--|									LIBRARIES									|
--+-----------------------------------------------------------------------------+

library ieee;
use ieee.std_logic_1164.all;


--+-----------------------------------------------------------------------------+
--|									ENTITY   									|
--+-----------------------------------------------------------------------------+

entity pcidec is
port (

   	-- General 
    clk_i       	: in std_logic;
   	nrst_i       	: in std_logic;
	-- pci 
	ad_i			: in std_logic_vector(31 downto 0);
	cbe_i			: in std_logic_vector(3 downto 0);
	idsel_i		  	: in std_logic;
	-- control
	bar0_i			: in std_logic_vector(31 downto 25);
	memEN_i			: in std_logic;
	pciadrLD_i	   	: in std_logic;
	adrcfg_o		: out std_logic;
	adrmem_o 		: out std_logic;
	adr_o			: out std_logic_vector(24 downto 1);
	cmd_o			: out std_logic_vector(3 downto 0)
	
);   
end pcidec;


architecture rtl of pcidec is


--+-----------------------------------------------------------------------------+
--|									COMPONENTS									|
--+-----------------------------------------------------------------------------+
--+-----------------------------------------------------------------------------+
--|									CONSTANTS  									|
--+-----------------------------------------------------------------------------+
--+-----------------------------------------------------------------------------+
--|									SIGNALS   									|
--+-----------------------------------------------------------------------------+

  	signal adr			: std_logic_vector(31 downto 0);
  	signal cmd			: std_logic_vector(3 downto 0);
  	signal idsel_s		: std_logic;
	signal a1			: std_logic;

begin

    --+-------------------------------------------------------------------------+
	--|  Load PCI Signals														|
    --+-------------------------------------------------------------------------+
	
	PCILD: process( nrst_i, clk_i, ad_i, cbe_i, idsel_i )
	begin

		if( nrst_i = '0' ) then 
			adr <= ( others => '1' );
			cmd <= ( others => '1' );
			idsel_s <= '0';
		elsif( rising_edge(clk_i) ) then 
		
			if ( pciadrLD_i = '1' ) then
		
				adr <= ad_i;
				cmd <= cbe_i;
				idsel_s <= idsel_i;
				
			end if;
		end if;
		
	end process PCILD;


	

    --+-------------------------------------------------------------------------+
    --|  Decoder																|
    --+-------------------------------------------------------------------------+

	adrmem_o <= '1' when (  ( memEN_i = '1' ) 
						and ( adr(31 downto 25) = bar0_i ) 
					    and ( adr(1 downto 0) = "00" ) 
					    and ( cmd(3 downto 1) = "011" )  )
					else '0';
					
	adrcfg_o <= '1' when (  ( idsel_s = '1' ) 
					    and ( adr(1 downto 0) = "00" ) 
					    and ( cmd(3 downto 1) = "101" )  )
					else '0';
	

    --+-------------------------------------------------------------------------+
    --|  Adress A(1)															|
    --+-------------------------------------------------------------------------+
	
	a1 <= not ( cbe_i(3) and cbe_i(2) );


    --+-------------------------------------------------------------------------+
	--|  Other outs																|
    --+-------------------------------------------------------------------------+

	adr_o <= adr(24 downto 2) & a1;
	cmd_o <= cmd;


end rtl;
