--+-------------------------------------------------------------------------------------------------+
--|																									|
--|  File:			pcidmux.vhd                                                          			|
--|																									|
--|  Project:		pci32tlite_oc 																	|
--|																									|
--|  Description: 	Multiplex data wb <-> regs <-> pci												|
--|					Multiplex data D16 whisbone <-> D32 PCI. 	                                    |
--|	 																								|
--+-------------------------------------------------------------------------------------------------+
--|																									|
--|  Revision history :																				|
--|  Date 		  Version	Author	Description														|
--|  2005-05-13   R00A00    PAU		First alfa revision	(eng)										|
--|																									|
--|  To do:		 																					|
--|																									|
--+-------------------------------------------------------------------------------------------------+
--+-----------------------------------------------------------------+
--| 																|
--|  Copyright (C) 2005 Peio Azkarate, peio@opencores.org   		| 
--| 																|
--|  This source file may be used and distributed without     		|
--|  restriction provided that this copyright statement is not		|
--|  removed from the file and that any derivative work contains	|
--|  the original copyright notice and the associated disclaimer.	|
--|                                                              	|
--|  This source file is free software; you can redistribute it     |
--|  and/or modify it under the terms of the GNU Lesser General     |
--|  Public License as published by the Free Software Foundation;   |
--|  either version 2.1 of the License, or (at your option) any     |
--|  later version.                                                 |
--| 																|
--|  This source is distributed in the hope that it will be         |
--|  useful, but WITHOUT ANY WARRANTY; without even the implied     |
--|  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR        |
--|  PURPOSE.  See the GNU Lesser General Public License for more   |
--|  details.                                                       |
--| 																|
--|  You should have received a copy of the GNU Lesser General      |
--|  Public License along with this source; if not, download it     |
--|  from http://www.opencores.org/lgpl.shtml                       |
--| 																|
--+-----------------------------------------------------------------+ 


--+-----------------------------------------------------------------------------+
--|									LIBRARIES									|
--+-----------------------------------------------------------------------------+

library ieee;
use ieee.std_logic_1164.all;


--+-----------------------------------------------------------------------------+
--|									ENTITY   									|
--+-----------------------------------------------------------------------------+

entity pcidmux is
port (

   	-- General 
    clk_i       	: in std_logic;
   	nrst_i       	: in std_logic;
	--  
	d_io			: inout std_logic_vector(31 downto 0);
	pcidatout_o		: out std_logic_vector(31 downto 0);
	--
	pcidOE_i	   	: in std_logic;
	wbdatLD_i   	: in std_logic;
	wbrgdMX_i   	: in std_logic;
	wbd16MX_i   	: in std_logic;	
	--
	wb_dat_i		: in std_logic_vector(15 downto 0);
	wb_dat_o		: out std_logic_vector(15 downto 0);
	rg_dat_i		: in std_logic_vector(31 downto 0);
	rg_dat_o		: out std_logic_vector(31 downto 0)
		
);   
end pcidmux;


architecture rtl of pcidmux is


--+-----------------------------------------------------------------------------+
--|									COMPONENTS									|
--+-----------------------------------------------------------------------------+
--+-----------------------------------------------------------------------------+
--|									CONSTANTS  									|
--+-----------------------------------------------------------------------------+
--+-----------------------------------------------------------------------------+
--|									SIGNALS   									|
--+-----------------------------------------------------------------------------+

  	signal pcidatin		: std_logic_vector(31 downto 0);
  	signal pcidatout	: std_logic_vector(31 downto 0);
  	signal wb_dat_is	: std_logic_vector(15 downto 0);

begin

    --+-------------------------------------------------------------------------+
	--|  Load Whisbone Datain													|
    --+-------------------------------------------------------------------------+
	
	WBDATLD: process( nrst_i, clk_i, wbdatLD_i, wb_dat_i )
	begin

		if( nrst_i = '0' ) then 
			wb_dat_is <= ( others => '1' );
		elsif( rising_edge(clk_i) ) then 
		
			if ( wbdatLD_i = '1' ) then
		
				wb_dat_is <= wb_dat_i;
				
			end if;
			
		end if;
		
	end process WBDATLD;


    --+-------------------------------------------------------------------------+
	--|  PCI data Triestate														|
    --+-------------------------------------------------------------------------+

	pcidatin <= d_io;
	d_io <= pcidatout when ( pcidOE_i = '1' ) else ( others => 'Z' );


    --+-------------------------------------------------------------------------+
	--|  Multiplex Registers and Whisbone databus								|
	--|  PCI(Little endian) to WB(Big endian)									|
	--|  PCI CFG REGS Little endian												|	
    --+-------------------------------------------------------------------------+
	
	pcidatout(31 downto 24) <= wb_dat_is(7 downto 0) when ( wbrgdMX_i = '1' ) else rg_dat_i(31 downto 24);
	pcidatout(23 downto 16) <= wb_dat_is(15 downto 8) when ( wbrgdMX_i = '1' ) else rg_dat_i(23 downto 16);
	pcidatout(15 downto 8) <= wb_dat_is(7 downto 0) when ( wbrgdMX_i = '1' ) else rg_dat_i(15 downto 8);
	pcidatout(7 downto 0) <= wb_dat_is(15 downto 8) when ( wbrgdMX_i = '1' ) else rg_dat_i(7 downto 0);
	
	pcidatout_o <= pcidatout;
	

    --+-------------------------------------------------------------------------+
	--|  Route PCI data toward Registers and Whisbone							|
    --+-------------------------------------------------------------------------+
	
	rg_dat_o <= pcidatin;
	
	wb_dat_o(15 downto 8) <= pcidatin(23 downto 16) when ( wbd16MX_i = '1' ) else pcidatin(7 downto 0);
	wb_dat_o(7 downto 0) <= pcidatin(31 downto 24) when ( wbd16MX_i = '1' ) else pcidatin(15 downto 8);
	

end rtl;
